package cz.mendelu.dnaAnalyser.analyse.palindrome;

import cz.mendelu.dnaAnalyser.analyse.palindrome.detector.AtatPalindromeDetector;
import cz.mendelu.dnaAnalyser.analyse.palindrome.model.PalindromeModel;
import cz.mendelu.dnaAnalyser.batch.BatchService;
import cz.mendelu.dnaAnalyser.sequence.NucleicBuffer;
import cz.mendelu.dnaAnalyser.sequence.data.SequenceData;
import cz.mendelu.dnaAnalyser.sequence.data.SequenceDataRepository;
import lombok.Data;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class PalindromeAnalyseService {

    private static final Logger LOG = LoggerFactory.getLogger(PalindromeAnalyseService.class);

    private List<PalindromeModel> palindromes = new LinkedList<>();

    private boolean cycleMode = false;

    //@Autowired
    //private PalindromeDataSessionService palindromDataSessionService;

    @Autowired
    private PalindromeAnalyseRepository palindromeRepository;

    @Autowired
    private SequenceDataRepository sequenceDataRepository;

    @Autowired
    private BatchService batchService;

    public PalindromeAnalyse executePalindromeAnalyse(PalindromeAnalyse palindromeAnalyse) {
        palindromeAnalyse = palindromeRepository.save(palindromeAnalyse);
        batchService.createBatch(palindromeAnalyse.getOwner(), palindromeAnalyse);
   /*             .execute(analyse->{

                    Sequence sequence = analyse.getSequence();
                    SequenceData sequenceData = sequenceDataRepository.load(sequence);

                    //inicializace proměnných potřebných pro algoritmus
                    int size = analyse.getSize();
                    boolean atatFilter = analyse.isAtatFilter();
                    boolean cycle = analyse.isCycle();
                    String mismatcheString = analyse.getMismatches();
                    String spacerString = analyse.getSpacers();

                    NumberRange mm = new NumberRange(mismatcheString); //prevedu mismatche a spacer
                    NumberRange sp = new NumberRange(spacerString);

                    final SortedSet<Integer> spacers = sp.getValues();
                    final SortedSet<Integer> mismatches = mm.getValues();

                    SqlSession sqlSession = palindromDataSessionService.openDataSession(analyse);

                    analyse.finishNow();
                    sqlSession.commit();
                    analyse = palindromeRepository.save(analyse); //na konci uložím
                    return analyse;
                }).start();

*/
        return palindromeAnalyse;
    }

    private boolean isSubSequenceToCompare(SequenceData sequence, int nextSequencePosition, int size, boolean cycleMode) {
        return (cycleMode)
                ? true
                : sequence.getLength() > nextSequencePosition + size;
    }

    private NucleicBuffer getSubSequence(SequenceData sequence, int nextSequencePosition, int size) {
        return sequence.getBuffer(nextSequencePosition, size);
    }

    public List<PalindromeModel> findPalindromes(final int size, SortedSet<Integer> spacers, SortedSet<Integer> mismatches, SequenceData sequence, boolean cycleMode, boolean atatFilter) {
        Integer nextSequencePosition, mismatche;
        NucleicBuffer testedSequence, oppositSequence, nextSequence;

        int searchToPosition = sequence.getLength();
        for (int i = 0; i < searchToPosition; i++) { // v cyklu prochazim cely genom

            testedSequence = getSubSequence(sequence, i, size); //vezme se část sekvence z celé sekvence a ta se testuje   

            LOG.debug("tested = {} ", testedSequence);
            if (atatFilter && AtatPalindromeDetector.isAtat(testedSequence)) continue;

            oppositSequence = testedSequence.revert(); //otočí se testovaná podsekvence
           // LOG.debug("OppositSeq = {} ", oppositSequence.getLength());

            for (int spacer : spacers) { // Postupne natahuji mezeru a porovnavam shodu
                nextSequencePosition = i + size + spacer;

                 if (isSubSequenceToCompare(sequence, nextSequencePosition, size, cycleMode)) { //zde je problém, neprochází
                nextSequence = getSubSequence(sequence, nextSequencePosition, size);

                if (atatFilter && AtatPalindromeDetector.isAtat(testedSequence)) continue;
                     LOG.debug("opposite = {} ", oppositSequence);
                     LOG.debug("next = {} ", nextSequence);

                mismatche = oppositSequence.mismatches(nextSequence); // mismatche null.. proč??? ZDE JE HLAVNÍ problém
                LOG.debug("mismatche = {} ", mismatche);
                if (mismatche != null && mismatches.contains(mismatche)) { // Pokud jsem nalezl odpovidaji sekvenci pridam ji do vysledku

                PalindromeModel p = (new PalindromeModel(sequence, i, size, spacer, mismatche));
                palindromes.add(p);
                break;
                    }
                }
            }
        }
        return palindromes;
    }



        @Data
        private static class Palindrome {
            private SequenceData sequenceData;
            private int position;
            private int mismatche;
            private int spacer;
            private int size;

            public Palindrome(SequenceData sequenceData, int position, int mismatche, int spacer, int size) {
                this.sequenceData = sequenceData;
                this.position = position;
                this.mismatche = mismatche;
                this.spacer = spacer;
                this.size = size;
            }
        }

    }
