package cz.mendelu.dnaAnalyser.batch;

import cz.mendelu.dnaAnalyser.jwt.JwtAuthenticationUtils;
import cz.mendelu.dnaAnalyser.utils.controller.DeleteMapping;
import cz.mendelu.dnaAnalyser.utils.controller.GetMapping;
import cz.mendelu.dnaAnalyser.utils.exception.NotFoundException;
import cz.mendelu.dnaAnalyser.utils.swagger.ApiPaginationParams;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;
import java.util.UUID;

@RestController
@Api(tags = "api/batch")
@RequestMapping("api/batch")
public class BatchController {

    @Autowired
    private BatchService batchService;

    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    @PreAuthorize("isAuthenticated()")
    @ApiPaginationParams
    @ApiOperation("Get page with batches.")
    public Iterable<Batch> getAllBatches(){
        return batchService.findAllBatches();
    }

    @GetMapping("/{entityClass}")
    @ResponseStatus(HttpStatus.OK)
    @PreAuthorize("isAuthenticated()")
    @ApiPaginationParams
    @ApiOperation("Get page with batches by entity class.")
    public Iterable<Batch> getBatchesByClass(
            @PathVariable String entityClass){
        return batchService.findAllByEntityClassAndOwner(entityClass);
    }

    @GetMapping("/{entityClass}/{entityId}")
    @ResponseStatus(HttpStatus.OK)
    @PreAuthorize("isAuthenticated()")
    @ApiOperation("Get one batch by entity class and UUID.")
    public Batch getBatchByEntityClassAndEntityId(
            @PathVariable String entityClass,
            @PathVariable UUID entityId){
        return batchService
                .findBatchByEntityClassAndEntityId(entityClass, entityId);
    }

    @DeleteMapping("/{entityClass}/{entityId}")
    @ResponseStatus(HttpStatus.OK)
    @PreAuthorize("isAuthenticated()")
    @ApiOperation("Delete one batch by entity class and UUID.")
    public void deleteBatchByEntityClassAndEntityId(
            @PathVariable String entityClass,
            @PathVariable UUID entityId){
        batchService.deleteBatchByEntityClassAndEntityId(entityClass, entityId);
    }
}
