package cz.mendelu.dnaAnalyser.sequence;

import cz.mendelu.dnaAnalyser.sequence.data.SequenceData;
import cz.mendelu.dnaAnalyser.sequence.data.SequenceDataService;
import cz.mendelu.dnaAnalyser.sequence.inport.ImportSequenceService;
import cz.mendelu.dnaAnalyser.sequence.nucleotide.Nucleotide;
import cz.mendelu.dnaAnalyser.utils.controller.TagFilterParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.Collections;
import java.util.EnumMap;
import java.util.Set;
import java.util.UUID;

/**
 * Created by Jan Kolomazník on 18.7.17.
 */
@Service
public class SequenceService {

    @Autowired
    ImportSequenceService importSequenceService;

    @Autowired
    ModelSequenceService modelSequenceService;

    @Autowired
    SequenceDataService sequenceDataService;

    public Page<Sequence> findAll(Pageable pageable, TagFilterParam tagFilterParam) {
        Set<String> tags = (tagFilterParam != null)
                ? tagFilterParam.getTags()
                : Collections.emptySet();
        return modelSequenceService.findAll(pageable, tags);
    }

    public Sequence findOne(UUID id) {
        return modelSequenceService.findOne(id);
    }

    public String getNcbi(UUID id) {
        return modelSequenceService.getNcbi(id);
    }

    public String getName(UUID id) {
        return modelSequenceService.getName(id);
    }


    public Sequence importPlainFormatFromText(Sequence sequence, byte[] string) {
        return importSequenceService.importPlainFormatFromText(sequence, string);
    }

    public Sequence importPlainFormatFromFile(Sequence sequence, MultipartFile file) {
        return importSequenceService.importPlainFormatFromFile(sequence, file);
    }

    public Sequence importFASTAFormatFromFile(Sequence sequence, MultipartFile file) {
        return importSequenceService.importFASTAFormatFromFile(sequence, file);
    }

    public Sequence importFASTAFormatFromText(Sequence sequence, byte[] string) {
        return importSequenceService.importFASTAFormatFromText(sequence, string);
    }

    public Sequence importNCBI(Sequence sequence, String ncbiId) {
        return importSequenceService.importNCBI(sequence, ncbiId);
    }

    public Sequence modifyTags(UUID id, Set<String> tags) {
        Sequence sequence = modelSequenceService.findOne(id);
        sequence.setTags(tags);
        modelSequenceService.save(sequence);
        return sequence;
    }

    public void delete(UUID id) {
        modelSequenceService.delete(id);
    }

    public Page<String> findAllTags(Pageable pageable) {
        return modelSequenceService.findAllTags(pageable);
    }

    // FIXME Duplicitní funkcionalita, nutno refatorovat!!!
    public Sequence updateNucleicCounts(UUID id) {
        Sequence sequence = modelSequenceService.findOne(id);
        SequenceData sequenceData = sequenceDataService.findSequenceBuffer(id);
        EnumMap<Nucleotide, Integer> nucleicCounts = new EnumMap<>(Nucleotide.class);
        for(Nucleotide n : sequenceData) {
            int i = nucleicCounts.getOrDefault(n, 0);
            nucleicCounts.put(n, i + 1);
        }
        sequence.setNucleicCounts(nucleicCounts);
        modelSequenceService.save(sequence);
        return sequence;
    }
}
