package cz.mendelu.dnaAnalyser.sequence.annotation;


import cz.mendelu.dnaAnalyser.analyse.g4hunter.G4Hunter;
import cz.mendelu.dnaAnalyser.analyse.g4hunter.G4HunterController;
import cz.mendelu.dnaAnalyser.jwt.JwtAuthenticationUtils;
import cz.mendelu.dnaAnalyser.sequence.Sequence;
import cz.mendelu.dnaAnalyser.sequence.SequenceService;
import cz.mendelu.dnaAnalyser.sequence.annotation.feature.Feature;
import cz.mendelu.dnaAnalyser.utils.controller.PaginationParam;
import cz.mendelu.dnaAnalyser.utils.controller.Response;
import cz.mendelu.dnaAnalyser.utils.model.Identifiable;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.persistence.Column;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.*;

@RestController
@Api(tags = "api/sequence")
@RequestMapping("/annotation")
public class AnnotationController {

    @Data
    static class AnnotationCreateRequest {

        private UUID sequence;

        private String type;

        private Integer start;

        private Integer stop;
    }

    @Data
    static class AnnotationResponse implements Identifiable {

        public UUID id;

        public Annotation annotation;

        public Page<Map<String, Object>> features;
    }

    @Autowired
    private AnnotationService annotationService;

    @Autowired
    private SequenceService sequenceService;

    @PostMapping()
    @ResponseStatus(HttpStatus.OK)
    @PreAuthorize("isAuthenticated()")
    @ApiOperation("Create annotation from sequence")
    public Response<Annotation> getAnnotation(
            @ApiIgnore Authentication authentication,
            @RequestBody @Valid AnnotationController.AnnotationCreateRequest request) {
        Sequence sequence = sequenceService.findOne(request.getSequence());
        Annotation annotation = annotationService.executeGetAnnotation(
                Annotation.builder()
                        .id(sequence.getId())
                        .type(request.getType())
                        .start(request.getStart())
                        .stop(request.getStop())
                        .sequence(sequence)
                        .owner(JwtAuthenticationUtils.toUser(authentication))
                        .build()
        );

        return Response.response(annotation);
    }

    @GetMapping("/{id}")
    @ResponseStatus(HttpStatus.OK)
    @PreAuthorize("isAuthenticated()")
    @ApiOperation("Get one annotation by ID")
    public Response<AnnotationResponse> getAnnotationByID(
            @PathVariable UUID id,
            PaginationParam paginationParam
    ) {
        Pageable pageable = paginationParam.toPageable(null);
        Annotation annotation = annotationService.findOne(id);
        Page<Map<String, Object>>  featureList = annotationService.retrieveFeatures(annotationService.getFeatures(annotation), pageable);
        AnnotationResponse annotationResponse = new AnnotationResponse();
        annotationResponse.setId(annotation.getId());
        annotationResponse.setAnnotation(annotation);
        annotationResponse.setFeatures(featureList);

        return Response.response(annotationResponse);

    }

}
